'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var p = require('path');
var fs = require('fs');
var requireFromString = require('require-from-string');

module.exports = {
  transformAndRequire,
  getReplacement,
  replace,
  resolveModuleContents,
  isCodegenComment,
  isPropertyCall,
  looksLike
};

function transformAndRequire(_ref, babel) {
  var code = _ref.code,
      fileOpts = _ref.fileOpts,
      _ref$fileOpts = _ref.fileOpts,
      filename = _ref$fileOpts.filename,
      plugins = _ref$fileOpts.plugins,
      presets = _ref$fileOpts.presets;

  /**
   * Transform passed in code to generated code
   */
  var _babel$transform = babel.transform(code, {
    filename,
    plugins,
    presets
  }),
      transformedCode = _babel$transform.code;

  /**
   * Execute the transformed code, as if it were required
   */


  var module = requireFromString(transformedCode, fileOpts.filename);

  /**
   * Allow for es modules (default export)
   */
  return module && module.__esModule ? module.default : module;
}

function getReplacement(_ref2, babel) {
  var code = _ref2.code,
      fileOpts = _ref2.fileOpts,
      _ref2$args = _ref2.args,
      args = _ref2$args === undefined ? [] : _ref2$args;

  /**
   * Execute the code string to get the exported module
   */
  var module = transformAndRequire({ code, fileOpts }, babel);

  /**
   * If a function is epxorted, call it with args
   */
  if (typeof module === 'function') {
    module = module.apply(undefined, args);
  } else if (args.length) {
    throw new Error(`codegen module (${p.relative(process.cwd(), fileOpts.filename)}) cannot accept arguments because it does not export a function. You passed the arguments: ${args.join(', ')}`);
  }

  /**
   * Convert whatever we got now (hopefully a string) into AST form
   */
  if (typeof module !== 'string') {
    throw new Error('codegen: Must module.exports a string.');
  }
  return babel.template(module, _extends({
    preserveComments: true,
    placeholderPattern: false
  }, fileOpts.parserOpts, {
    sourceType: 'module'
  }))();
}

function applyReplacementToPath(replacement, path) {
  if (!replacement) {
    path.remove();
  } else if (Array.isArray(replacement)) {
    path.replaceWithMultiple(replacement);
  } else {
    path.replaceWith(replacement);
  }
}

function replace(_ref3, babel) {
  var path = _ref3.path,
      code = _ref3.code,
      fileOpts = _ref3.fileOpts,
      args = _ref3.args;

  var replacement = getReplacement({ code, args, fileOpts }, babel);
  applyReplacementToPath(replacement, path);
}

function resolveModuleContents(_ref4) {
  var filename = _ref4.filename,
      module = _ref4.module;

  var resolvedPath = p.resolve(p.dirname(filename), module);
  var code = fs.readFileSync(require.resolve(resolvedPath));
  return { code, resolvedPath };
}

function isCodegenComment(comment) {
  var normalisedComment = comment.value.trim().split(' ')[0].trim();
  return normalisedComment.startsWith('codegen') || normalisedComment.startsWith('@codegen');
}

function isPropertyCall(path, name) {
  return looksLike(path, {
    node: {
      type: 'CallExpression',
      callee: {
        property: { name }
      }
    }
  });
}

function looksLike(a, b) {
  return a && b && Object.keys(b).every(function (bKey) {
    var bVal = b[bKey];
    var aVal = a[bKey];
    if (typeof bVal === 'function') {
      return bVal(aVal);
    }
    return isPrimitive(bVal) ? bVal === aVal : looksLike(aVal, bVal);
  });
}

function isPrimitive(val) {
  // eslint-disable-next-line
  return val == null || /^[sbn]/.test(typeof val);
}

/*
eslint
  complexity: ["error", 8],
  import/no-unassigned-import: "off",
  import/no-dynamic-require: "off",
*/